@@ -2,7 +2,7 @@
 //
 // MakeCall.cxx
 //
-// Copyright (c) 2007-2010, Jan Willamowius
+// Copyright (c) 2007-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -41,7 +41,7 @@ MakeCallEndPoint::MakeCallEndPoint() : Singleton<MakeCallEndPoint>("MakeCallEndP
 	// Establish link with gatekeeper
 	H323TransportUDP * rasChannel;
 	if (GkConfig()->GetString("CTI::MakeCall", "Interface", "").IsEmpty())
-		rasChannel = new H323TransportUDP(*this, INADDR_ANY, 1722);
+		rasChannel = new H323TransportUDP(*this, GNUGK_INADDR_ANY, 1722);
 	else {
 		PIPSocket::Address interfaceAddress(GkConfig()->GetString("CTI::MakeCall", "Interface", ""));
 		rasChannel = new H323TransportUDP(*this, interfaceAddress);
@@ -71,7 +71,7 @@ void MakeCallEndPoint::ThirdPartyMakeCall(const PString & user1, const PString &
 	AddDestination(newToken, user2);
 }
 
-PBoolean MakeCallEndPoint::IsRegisteredWithGk(void) const
+PBoolean MakeCallEndPoint::IsRegisteredWithGk() const
 {
 	return isRegistered;
 }	
@@ -131,7 +131,7 @@ void MakeCallEndPoint::OnConnectionEstablished(H323Connection & connection,
 {
 	// find second party by call token
 	PString second_party = GetDestination(token);
-	PTRACE(1, "MakeCallEndpoint: Transfering call to 2nd party " << second_party);
+	PTRACE(1, "MakeCallEndpoint: Transferring call to 2nd party " << second_party);
 	if (transferMethod == "H.450.2") {
 		PTRACE(3, "MakeCallEndpoint: Using H.450.2 to transfer call");
 		connection.TransferCall(second_party);
