@@ -3,7 +3,7 @@
 // Neighboring System for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2010, Jan Willamowius
+// Copyright (c) 2004-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -54,7 +54,7 @@ void SetCryptoGkTokens(H225_ArrayOf_CryptoH323Token & cryptoTokens, const PStrin
 	cryptoTokens.SetSize(0);
 
 	H235AuthSimpleMD5 auth;
-	// avoid copying for thread-safely
+	// avoid copying for thread-safety
 	auth.SetLocalId((const char *)id);
 	auth.SetPassword((const char *)password);
 	H225_ArrayOf_ClearToken dumbTokens;
@@ -62,19 +62,18 @@ void SetCryptoGkTokens(H225_ArrayOf_CryptoH323Token & cryptoTokens, const PStrin
 	auth.PrepareTokens(dumbTokens, newCryptoTokens);
 	H225_CryptoH323Token_cryptoEPPwdHash & cryptoEPPwdHash = newCryptoTokens[0];
 
-	// Create the H.225 GK crypto token
+	// Create the H.225 GK crypto token with the hash calculated for the EP token
 	H225_CryptoH323Token * finalCryptoToken = new H225_CryptoH323Token;
 	finalCryptoToken->SetTag(H225_CryptoH323Token::e_cryptoGKPwdHash);
 	H225_CryptoH323Token_cryptoGKPwdHash & cryptoGKPwdHash = *finalCryptoToken;
 
 	// Set the token data that actually goes over the wire
-	cryptoGKPwdHash.m_gatekeeperId = Toolkit::GKName();
+	cryptoGKPwdHash.m_gatekeeperId = id;
 	cryptoGKPwdHash.m_timeStamp = cryptoEPPwdHash.m_timeStamp;
 	cryptoGKPwdHash.m_token.m_algorithmOID = OID_MD5;
 	cryptoGKPwdHash.m_token.m_hash = cryptoEPPwdHash.m_token.m_hash;
 	
-	if (finalCryptoToken != NULL)
-		cryptoTokens.Append(finalCryptoToken);
+	cryptoTokens.Append(finalCryptoToken);
 }
 
 class OldGK : public Neighbor {
@@ -197,6 +196,8 @@ Neighbor::Neighbor()
 	m_rasSrv = RasServer::Instance();
 	m_keepAliveTimer = GkTimerManager::INVALID_HANDLE;
 	m_keepAliveTimerInterval = 0;
+	m_H46018Server = false;
+	m_H46018Client = false;
 }
 
 Neighbor::~Neighbor()
@@ -223,6 +224,18 @@ PIPSocket::Address Neighbor::GetIP() const
 	return m_ip;
 }
 
+WORD Neighbor::GetPort() const
+{
+	if (m_dynamic) {
+		PIPSocket::ClearNameCache();
+		// Retrieve the ip address at this time
+		if (!GetTransportAddress(m_name, GK_DEF_UNICAST_RAS_PORT, m_ip, m_port)) {
+			PTRACE(1, "NB\tCan't get neighbor port for " << m_name);
+		}
+	}
+	return m_port;
+}
+
 H225_LocationRequest & Neighbor::BuildLRQ(H225_RasMessage & lrq_ras, WORD seqnum, const H225_ArrayOf_AliasAddress & dest)
 {
 	lrq_ras.SetTag(H225_RasMessage::e_locationRequest);
@@ -245,9 +258,17 @@ H225_LocationRequest & Neighbor::BuildLRQ(H225_RasMessage & lrq_ras, WORD seqnum
 	lrq.IncludeOptionalField(H225_LocationRequest::e_sourceInfo);
 	lrq.m_sourceInfo.SetSize(1);
 	H323SetAliasAddress(Toolkit::GKName(), lrq.m_sourceInfo[0], H225_AliasAddress::e_h323_ID);
-	
-	if (m_externalGK)
-	   m_rasSrv->GetGkClient()->SetNBPassword(lrq, Toolkit::GKName());
+
+	// TODO: is this right ?
+	if (m_externalGK) {
+		m_rasSrv->GetGkClient()->SetNBPassword(lrq, Toolkit::GKName());
+	}
+
+	if (!m_sendPassword.IsEmpty()) {
+		lrq.IncludeOptionalField(H225_LocationRequest::e_cryptoTokens);
+		lrq.m_cryptoTokens.SetSize(1);
+		SetCryptoGkTokens(lrq.m_cryptoTokens, m_sendAuthUser, m_sendPassword);
+	}
 
 	if (m_forwardHopCount >= 1) { // what if set hopCount = 1?
 		lrq.IncludeOptionalField(H225_LocationRequest::e_hopCount);
@@ -265,8 +286,10 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 	m_name = config->GetString(section, "Host", "");
 	m_dynamic = Toolkit::AsBool(config->GetString(section, "Dynamic", "0"));
 	m_externalGK = false;
+	m_authUser = config->GetString(section, "AuthUser", m_gkid);	// defaults to GatekeeperIdentifier
 	m_password = Toolkit::Instance()->ReadPassword(section, "Password");	// checking incomming password in LRQ (not implemented, yet)
-	m_sendPassword = Toolkit::Instance()->ReadPassword(section, "SendPassword", "");	// password to send to neighbor
+	m_sendAuthUser = config->GetString(section, "SendAuthUser", Toolkit::GKName());	// defaults to own GatekeeperId
+	m_sendPassword = Toolkit::Instance()->ReadPassword(section, "SendPassword");	// password to send to neighbor
 
 	if (!m_dynamic && !GetTransportAddress(m_name, GK_DEF_UNICAST_RAS_PORT, m_ip, m_port))
 		return false;
@@ -279,7 +302,7 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 		PStringArray p(sprefixes[i].Tokenise(":=", false));
 		m_sendPrefixes[p[0]] = (p.GetSize() > 1) ? p[1].AsInteger() : 1;
 	}
-
+ 
 	PString salias(config->GetString(section, "SendAliases", ""));
 	PStringArray defs(salias.Tokenise(",", FALSE));
 	m_sendAliases.SetSize(0);
@@ -311,15 +334,19 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 			m_sendAliases.AppendString(defs[i]);
 		}
 	}
-
+ 
 	PString aprefix(config->GetString(section, "AcceptPrefixes", "*"));
 	m_acceptPrefixes = PStringArray(aprefix.Tokenise(",", false));
 	if (m_keepAliveTimer != GkTimerManager::INVALID_HANDLE)
 		Toolkit::Instance()->GetTimerManager()->UnregisterTimer(m_keepAliveTimer);
 #ifdef HAS_H46018
-	if (Toolkit::AsBool(config->GetString(section, "UseH46018", "0"))) {
+	if (Toolkit::AsBool(config->GetString(section, "H46018Client", "0"))) {
+		m_H46018Client = true;
 		SetH46018GkKeepAliveInterval(29);	// start with every 29 seconds
 	}
+	if (Toolkit::AsBool(config->GetString(section, "H46018Server", "0"))) {
+		m_H46018Server = true;
+	}
 #endif
 
 	SetForwardedInfo(section);
@@ -336,7 +363,7 @@ bool Neighbor::SetProfile(const PString & id, const PString & type)
 void Neighbor::SendH46018GkKeepAlive(GkTimer* timer)
 {
 #ifdef HAS_H46018
-	// send SCI to open the pinhole
+	// send SCI to open the pinhole to neighbor GK
 	H225_RasMessage sci_ras;
 	sci_ras.SetTag(H225_RasMessage::e_serviceControlIndication);
 	H225_ServiceControlIndication & sci = sci_ras;
@@ -356,7 +383,7 @@ void Neighbor::SendH46018GkKeepAlive(GkTimer* timer)
 	if (!m_sendPassword.IsEmpty()) {
 		sci.IncludeOptionalField(H225_ServiceControlIndication::e_cryptoTokens);
 		sci.m_cryptoTokens.SetSize(1);
-		SetCryptoGkTokens(sci.m_cryptoTokens, Toolkit::GKName(), m_sendPassword);
+		SetCryptoGkTokens(sci.m_cryptoTokens, m_sendAuthUser, m_sendPassword);
 	}
 	m_rasSrv->SendRas(sci_ras, GetIP(), m_port);
 #endif
@@ -437,7 +464,7 @@ PrefixInfo Neighbor::GetPrefixInfo(const H225_ArrayOf_AliasAddress & aliases, H2
 	return PrefixInfo(0, (short)iter->second);
 }
 
-bool Neighbor::OnSendingLRQ(H225_LocationRequest &)
+bool Neighbor::OnSendingLRQ(H225_LocationRequest & lrq)
 {
 	return true;
 }
@@ -502,6 +529,62 @@ bool Neighbor::CheckReply(RasMsg *ras) const
 		: false;
 }
 
+bool Neighbor::Authenticate(RasMsg *ras) const
+{
+	if (m_password.IsEmpty()) {
+		return true;	// no password, no check needed
+	} else {
+		// check tokens
+		H225_ArrayOf_CryptoH323Token tokens;
+		// check LRQs and SCIs
+		switch (ras->GetTag()) {
+			case H225_RasMessage::e_locationRequest:
+				{
+					H225_LocationRequest & lrq = (*ras)->m_recvRAS;
+					if (lrq.HasOptionalField(H225_LocationRequest::e_cryptoTokens))
+						tokens = lrq.m_cryptoTokens;
+				}
+				break;
+			case H225_RasMessage::e_serviceControlIndication:
+				{
+					H225_ServiceControlIndication & sci = (*ras)->m_recvRAS;
+					if (sci.HasOptionalField(H225_ServiceControlIndication::e_cryptoTokens))
+						tokens = sci.m_cryptoTokens;
+				}
+				break;
+			default:
+				break;
+		}
+
+		H235AuthSimpleMD5 authMD5;
+		PBYTEArray dummy;
+		authMD5.SetLocalId(m_authUser);
+		authMD5.SetPassword(m_password);
+		for (PINDEX i = 0 ; i < tokens.GetSize(); i++) {
+			H225_CryptoH323Token cryptoToken;
+			if (tokens[i].GetTag() == H225_CryptoH323Token::e_cryptoGKPwdHash) {
+				// convert the GKPwdHash into an EPPwdHash that H323Plus is able to check
+				H225_CryptoH323Token_cryptoGKPwdHash & cryptoGKPwdHash = tokens[i];
+				cryptoToken.SetTag(H225_CryptoH323Token::e_cryptoEPPwdHash);
+				H225_CryptoH323Token_cryptoEPPwdHash & cryptoEPPwdHash = cryptoToken;
+				H323SetAliasAddress(cryptoGKPwdHash.m_gatekeeperId, cryptoEPPwdHash.m_alias);
+				cryptoEPPwdHash.m_timeStamp = cryptoGKPwdHash.m_timeStamp;
+				cryptoEPPwdHash.m_token.m_algorithmOID = OID_MD5;
+				cryptoEPPwdHash.m_token.m_hash = cryptoGKPwdHash.m_token.m_hash;
+			} else {
+				cryptoToken = tokens[i];	// use other tokens as they are
+			}
+			if (authMD5.ValidateCryptoToken(cryptoToken, dummy) == H235Authenticator::e_OK) {
+				PTRACE(5, "Neighbor\tMD5 password match");
+				return true;
+			}
+		}
+
+		PTRACE(1, "Neighbor\tPassword required, but no match");
+		return false;	// no token found that allows access
+	}
+}
+ 
 bool Neighbor::IsAcceptable(RasMsg *ras) const
 {
 	if (ras->IsFrom(GetIP(), 0 /*m_port*/)) {
@@ -511,9 +594,11 @@ bool Neighbor::IsAcceptable(RasMsg *ras) const
 		H225_ArrayOf_AliasAddress & aliases = lrq.m_destinationInfo;
 		for (j = 0; j < aliases.GetSize(); ++j) {
 			H225_AliasAddress & alias = aliases[j];
-			for (i = 0; i < sz; ++i)
-				if (m_acceptPrefixes[i] == alias.GetTagName())
+			for (i = 0; i < sz; ++i) {
+				if (m_acceptPrefixes[i] == alias.GetTagName()) {
 					return true;
+				}
+			}
 			PString destination(AsString(alias, false));
 			int maxlen = 0;
 			for (i = 0; i < sz; ++i) {
@@ -527,9 +612,11 @@ bool Neighbor::IsAcceptable(RasMsg *ras) const
 			if (maxlen > 0)
 				return true;
 		}
-		for (i = 0; i < sz; ++i)
-			if (m_acceptPrefixes[i] == "*")
+		for (i = 0; i < sz; ++i) {
+			if (m_acceptPrefixes[i] == "*") {
 				return true;
+			}
+		}
 	}
 	return false;
 }
@@ -553,7 +640,7 @@ void Neighbor::SetForwardedInfo(const PString & section)
 // class OldGK
 bool OldGK::SetProfile(const PString & id, const PString & args)
 {
-	m_id = m_gkid = id;
+	m_authUser = m_id = m_gkid = id;
 	PStringArray cfg(args.Tokenise(";", true));
 	m_name = cfg[0].Trim();
 	m_sendPrefixes.clear();
@@ -601,26 +688,30 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const AdmissionRequest & re
 		lrq.m_canMapAlias = arq.m_canMapAlias;
 	}
 
-#ifdef HAS_H460
-	lrq.IncludeOptionalField(H225_LocationRequest::e_genericData); 
-	H225_ArrayOf_GenericData & data = lrq.m_genericData;
-	PINDEX lastPos = 0;
+	// must include callID to traversal servers and clients, no harm to always include it
+	if (arq.HasOptionalField(H225_AdmissionRequest::e_callIdentifier)) {
+		lrq.IncludeOptionalField(H225_LocationRequest::e_callIdentifier);
+		lrq.m_callIdentifier = arq.m_callIdentifier;
+	}
 
 #ifdef HAS_H46023
-	if (Toolkit::Instance()->IsH46023Enabled()) {
-		H460_FeatureStd std24 = H460_FeatureStd(24);
-		lastPos++;
-		data.SetSize(lastPos);
-		data[lastPos-1] = std24;
-	}
+    /// STD24  NAT Support
+    if (Toolkit::Instance()->IsH46023Enabled()) {
+        H460_FeatureStd std24 = H460_FeatureStd(24);
+        int sz = lrq.m_genericData.GetSize();
+        lrq.m_genericData.SetSize(sz+1);
+        lrq.m_genericData[sz] = std24;
+    }
 #endif
-
-	/// OID9  'Remote application info
-	H460_FeatureOID foid9 = H460_FeatureOID(OID9);
-	lastPos++;
-	data.SetSize(lastPos);
-	data[lastPos-1] = foid9;	
+#if 0   // Disable until we workout moving non-standard H.460 stuff to new compile directive. - SH
+     /// OID9  'Remote endpoint vendor info THIS IS "1.3.6.1.4.1.17090.0.9" NOT H.460.9	- SH
+     H460_FeatureOID foid9 = H460_FeatureOID(OID9);	 
+     int sz = lrq.m_genericData.GetSize();	 
+     lrq.m_genericData.SetSize(sz+1);	 
+     lrq.m_genericData[sz] = foid9;
 #endif
+     if (lrq.m_genericData.GetSize() > 0)
+          lrq.IncludeOptionalField(H225_LocationRequest::e_genericData);
 
 	return true;
 }
@@ -645,6 +736,13 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const SetupRequest & reques
 	
 	lrq.IncludeOptionalField(H225_LocationRequest::e_canMapAlias);
 	lrq.m_canMapAlias = TRUE;
+
+	// must include callID to traversal servers and clients, no harm to always include it
+	if (setup.HasOptionalField(H225_Setup_UUIE::e_callIdentifier)) {
+		lrq.IncludeOptionalField(H225_LocationRequest::e_callIdentifier);
+		lrq.m_callIdentifier = setup.m_callIdentifier;
+	}
+
 	return true;
 }
 
@@ -662,6 +760,7 @@ bool GnuGK::OnSendingLRQ(H225_LocationRequest & lrq, const FacilityRequest & /*r
 	
 	lrq.IncludeOptionalField(H225_LocationRequest::e_canMapAlias);
 	lrq.m_canMapAlias = TRUE;
+
 	return true;
 }
 
@@ -879,7 +978,10 @@ public:
 	bool Send(Neighbor * nb);
 	int GetReqNumber() const { return m_requests.size(); }
 	H225_LocationConfirm *WaitForDestination(int);
-	PString GetNeighborUsed() { return m_neighbor_used; }
+	PString GetNeighborUsed() const { return m_neighbor_used; }
+	bool IsTraversalClient() const { return m_h46018_client; }
+	bool IsTraversalServer() const { return m_h46018_server; }
+	bool IsTraversalZone() const { return m_h46018_client || m_h46018_server; }
 
 	// override from class RasRequester
 	virtual bool IsExpected(const RasMsg *) const;
@@ -902,9 +1004,10 @@ private:
 	const LRQFunctor & m_sendto;
 	RasMsg *m_result;
 	PString m_neighbor_used;
+	bool m_h46018_client, m_h46018_server;
 };
 
-LRQRequester::LRQRequester(const LRQFunctor & fun) : m_sendto(fun), m_result(0)
+LRQRequester::LRQRequester(const LRQFunctor & fun) : m_sendto(fun), m_result(0), m_h46018_client(false), m_h46018_server(false)
 {
 	AddFilter(H225_RasMessage::e_locationConfirm);
 	AddFilter(H225_RasMessage::e_locationReject);
@@ -951,15 +1054,17 @@ bool LRQRequester::Send(Neighbor * nb)
 	return true;
 }
 
-H225_LocationConfirm *LRQRequester::WaitForDestination(int timeout)
+H225_LocationConfirm * LRQRequester::WaitForDestination(int timeout)
 {
-	while (WaitForResponse(timeout))
-		if (m_result)
+	while (WaitForResponse(timeout)) {
+		if (m_result) {
 			break;
-		else
+		} else {
 			GetReply(); // ignore and increase iterator
+		}
+	}
 
-	return m_result ? &(H225_LocationConfirm &)(*m_result)->m_recvRAS : 0;
+	return m_result ? &(H225_LocationConfirm &)(*m_result)->m_recvRAS : NULL;
 }
 
 bool LRQRequester::IsExpected(const RasMsg *ras) const
@@ -974,12 +1079,12 @@ void LRQRequester::Process(RasMsg *ras)
 		Request & req = iter->second;
 		if (req.m_neighbor->CheckReply(ras) ||
 			Toolkit::AsBool(GkConfig()->GetString(LRQFeaturesSection, "AcceptNonNeighborLCF", "0"))) {
-			PTRACE(5,"NB\tReceived "<<ras->GetTagName()<<" message matched"
-				<<" pending LRQ for neighbor "<<req.m_neighbor->GetId()
-				<<':'<<req.m_neighbor->GetIP()
-				);
+			PTRACE(5, "NB\tReceived " << ras->GetTagName() << " message matched"
+				<< " pending LRQ for neighbor " << req.m_neighbor->GetId()
+				<< ':' << req.m_neighbor->GetIP() );
 			unsigned tag = ras->GetTag();
 			if (tag == H225_RasMessage::e_requestInProgress) {
+				// TODO: honor the delay specified in the RIP ?
 				if (H225_NonStandardParameter *param = ras->GetNonStandardParam()) {
 					int iec = Toolkit::iecUnknown;
 					if (param->m_nonStandardIdentifier.GetTag() == H225_NonStandardIdentifier::e_h221NonStandard) {
@@ -1003,6 +1108,13 @@ void LRQRequester::Process(RasMsg *ras)
 					m_result = ras;
 				AddReply(req.m_reply = ras);
 				m_neighbor_used = req.m_neighbor->GetId(); // record neighbor used
+				m_h46018_client = req.m_neighbor->IsH46018Client();
+				m_h46018_server = req.m_neighbor->IsH46018Server();
+				if (m_h46018_server) {
+					// if we are traversal server we must use the apparent RAS IP of the client
+					H225_LocationConfirm & lcf = (*ras)->m_recvRAS;
+					lcf.m_rasAddress = SocketToH225TransportAddr(req.m_neighbor->GetIP(), req.m_neighbor->GetPort());
+				}
 				if (m_result)
 					m_sync.Signal();
 			} else { // should be H225_RasMessage::e_locationReject
@@ -1010,9 +1122,8 @@ void LRQRequester::Process(RasMsg *ras)
 				delete ras;
 				ras = NULL;
 				if (req.m_count <= 0 && req.m_reply == 0) {
-					PTRACE(5,"NB\tLRQ rejected for neighbor "<<req.m_neighbor->GetId()
-						<<':'<<req.m_neighbor->GetIP()
-						);
+					PTRACE(5, "NB\tLRQ rejected for neighbor " << req.m_neighbor->GetId()
+						<< ':' << req.m_neighbor->GetIP() );
 					m_requests.erase(iter);
 					if (m_requests.empty())
 						RasRequester::Stop();
@@ -1087,6 +1198,8 @@ void NeighborList::OnReload()
 		const PString & nbid = cfgs.GetKeyAt(i);
 		PString type = cfgs.GetDataAt(i);
 		// make neighbor type caseless
+		if (PCaselessString(type) == "Generic")
+			type = "GnuGK";
 		if (PCaselessString(type) == "GnuGK")
 			type = "GnuGK";
 		if (PCaselessString(type) == "CiscoGK")
@@ -1115,7 +1228,13 @@ void NeighborList::OnReload()
 
 bool NeighborList::CheckLRQ(RasMsg *ras) const
 {
-     return find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsAcceptable), ras)) != m_neighbors.end();
+	List::const_iterator iter = m_neighbors.begin();
+	while (iter != m_neighbors.end()) {
+		if ((*iter)->IsAcceptable(ras) && (*iter)->Authenticate(ras))
+			return true;
+		++iter;
+	}
+	return false;
 }
 
 bool NeighborList::CheckIP(const PIPSocket::Address & addr) const
@@ -1123,6 +1242,21 @@ bool NeighborList::CheckIP(const PIPSocket::Address & addr) const
 	return find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsFrom), &addr)) != m_neighbors.end();
 }
 
+bool NeighborList::IsTraversalZone(const PIPSocket::Address & addr) const
+{
+	return find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsTraversalZone), &addr)) != m_neighbors.end();
+}
+
+bool NeighborList::IsTraversalClient(const PIPSocket::Address & addr) const
+{
+	return find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsTraversalClient), &addr)) != m_neighbors.end();
+}
+
+bool NeighborList::IsTraversalServer(const PIPSocket::Address & addr) const
+{
+	return find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsTraversalServer), &addr)) != m_neighbors.end();
+}
+
 PString NeighborList::GetNeighborIdBySigAdr(const H225_TransportAddress & sigAd)
 {
 	PIPSocket::Address ipaddr;
@@ -1140,15 +1274,13 @@ PString NeighborList::GetNeighborIdBySigAdr(const PIPSocket::Address & sigAd)
 {
 	// Attempt to find the neigbor in the list
 	List::iterator findNeighbor = find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsFrom), &sigAd));
-
 	if (findNeighbor == m_neighbors.end())
 	{
 		return PString::Empty();
 	}
-
 	return (*findNeighbor)->GetId();
 }
-
+ 
 PString NeighborList::GetNeighborGkIdBySigAdr(const PIPSocket::Address & sigAd)
 {
 	// Attempt to find the neigbor in the list
@@ -1165,13 +1297,12 @@ PString NeighborList::GetNeighborGkIdBySigAdr(const PIPSocket::Address & sigAd)
 PString NeighborList::GetNeighborGkIdBySigAdr(const H225_TransportAddress & sigAd)
 {
 	PIPSocket::Address ipaddr;
-
+ 
 	// Get the Neigbor IP address from the transport address
 	if (!GetIPFromTransportAddr(sigAd, ipaddr))
 	{
 		return PString::Empty();
 	}
-
 	return GetNeighborGkIdBySigAdr(ipaddr);
 }
 
@@ -1295,12 +1426,21 @@ bool NeighborPolicy::OnRequest(AdmissionRequest & arq_obj)
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
 #ifdef HAS_H460
-			if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData)) {
-			  H225_RasMessage ras;
-			  ras.SetTag(H225_RasMessage::e_locationConfirm);
-              H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
-			  con = *lcf;
-			  route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);	
+			if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone()) {
+				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
+				H225_RasMessage ras;
+				ras.SetTag(H225_RasMessage::e_locationConfirm);
+				H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
+				con = *lcf;
+				route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+				// set flag to use H.460.18 if neighbor is traversal server
+				if (request.IsTraversalClient()) {
+					// if we are the client, then the call goes to a traversal server
+					route.m_destEndpoint->SetTraversalRole(TraversalServer);
+				}
+				if (request.IsTraversalServer()) {
+                    route.m_destEndpoint->SetTraversalRole(TraversalClient);
+				}
 			}
 #endif
 			route.m_routeId = request.GetNeighborUsed();
@@ -1326,7 +1466,7 @@ bool NeighborPolicy::OnRequest(LocationRequest & lrq_obj)
 {
 	RasMsg *ras = lrq_obj.GetWrapper();
 	List::iterator iter = find_if(m_neighbors.begin(), m_neighbors.end(), bind2nd(mem_fun(&Neighbor::IsAcceptable), ras));
-	Neighbor *requester = (iter != m_neighbors.end()) ? *iter : 0;
+	Neighbor *requester = (iter != m_neighbors.end()) ? *iter : NULL;
 	int hopCount = 0;
 	if (requester) {
 		if (requester->ForwardLRQ() < 0) {
@@ -1382,6 +1522,24 @@ bool NeighborPolicy::OnRequest(LocationRequest & lrq_obj)
 		if (request.Send(m_neighbors, requester)) {
 			if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 				Route route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_H460
+				if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone()) {
+					// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
+					H225_RasMessage ras;
+					ras.SetTag(H225_RasMessage::e_locationConfirm);
+					H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
+					con = *lcf;
+					route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+					// set flag to use H.460.18 if neighbor is traversal server
+					if (request.IsTraversalClient()) {
+						// if we are the client, then the call goes to a traversal server
+						route.m_destEndpoint->SetTraversalRole(TraversalServer);
+					}
+					if (request.IsTraversalServer()) {
+                        route.m_destEndpoint->SetTraversalRole(TraversalClient);
+					}
+				}
+#endif
 				route.m_routeId = request.GetNeighborUsed();
 				route.m_flags |= Route::e_toNeighbor;
 				lrq_obj.AddRoute(route);
@@ -1411,6 +1569,24 @@ bool NeighborPolicy::OnRequest(SetupRequest & setup_obj)
 	if (request.Send(m_neighbors)) {
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_H460
+			if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone()) {
+				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
+				H225_RasMessage ras;
+				ras.SetTag(H225_RasMessage::e_locationConfirm);
+				H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
+				con = *lcf;
+				route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+				// set flag to use H.460.18 if neighbor is traversal server
+				if (request.IsTraversalClient()) {
+					// if we are the client, then the call goes to a traversal server
+					route.m_destEndpoint->SetTraversalRole(TraversalServer);
+				}
+				if (request.IsTraversalServer()) {
+                    route.m_destEndpoint->SetTraversalRole(TraversalClient);
+				}
+			}
+#endif
 			route.m_routeId = request.GetNeighborUsed();
 			route.m_flags |= Route::e_toNeighbor;
 			setup_obj.AddRoute(route);
@@ -1435,6 +1611,24 @@ bool NeighborPolicy::OnRequest(FacilityRequest & facility_obj)
 	if (request.Send(m_neighbors)) {
 		if (H225_LocationConfirm *lcf = request.WaitForDestination(m_neighborTimeout)) {
 			Route route(m_name, lcf->m_callSignalAddress);
+#ifdef HAS_H460
+			if (lcf->HasOptionalField(H225_LocationConfirm::e_genericData) || request.IsTraversalZone()) {
+				// create an EPRec to remember the NAT settings for H.460.18 (traversal zone) or H.460.23/.24 (genericData)
+				H225_RasMessage ras;
+				ras.SetTag(H225_RasMessage::e_locationConfirm);
+				H225_LocationConfirm & con = (H225_LocationConfirm &)ras;
+				con = *lcf;
+				route.m_destEndpoint = RegistrationTable::Instance()->InsertRec(ras);
+				// set flag to use H.460.18 if neighbor is traversal server
+				if (request.IsTraversalClient()) {
+					// if we are the client, then the call goes to a traversal server
+					route.m_destEndpoint->SetTraversalRole(TraversalServer);
+				}
+				if (request.IsTraversalServer()) {
+                    route.m_destEndpoint->SetTraversalRole(TraversalClient);
+				}
+			}
+#endif
 			route.m_routeId = request.GetNeighborUsed();
 			route.m_flags |= Route::e_toNeighbor;
 			facility_obj.AddRoute(route);
@@ -1449,7 +1643,7 @@ bool NeighborPolicy::OnRequest(FacilityRequest & facility_obj)
 #ifdef hasSRV
 class SRVPolicy : public AliasesPolicy {
 public:
-	SRVPolicy() { m_name = "SRV"; }
+	SRVPolicy();
 
 protected:
     virtual bool OnRequest(FacilityRequest &) { return false; }
@@ -1460,8 +1654,16 @@ protected:
 	virtual Route * CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly);
 	virtual Route * LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases);
 	virtual Route * LSLocalLookup(H225_ArrayOf_AliasAddress & aliases);
+
+	bool m_resolveNonLocalLRQs;
 };
 
+SRVPolicy::SRVPolicy()
+{
+	m_name = "SRV";
+	m_resolveNonLocalLRQs = Toolkit::AsBool(GkConfig()->GetString("Routing::SRV", "ResolveNonLocalLRQ", "0"));
+}
+
 Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
@@ -1559,7 +1761,7 @@ Route * SRVPolicy::LSLookup(RoutingRequest & request, H225_ArrayOf_AliasAddress
 	}
 	return NULL;
 }
-
+ 
 Route * SRVPolicy::LSLocalLookup(H225_ArrayOf_AliasAddress & aliases)
 {
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
@@ -1572,14 +1774,14 @@ Route * SRVPolicy::LSLocalLookup(H225_ArrayOf_AliasAddress & aliases)
 		// skip empty aliases or those without at-sign
 	    if ((alias.GetLength() == 0) || (at == P_MAX_INDEX))
 			continue;
-
+ 
 		// DNS SRV Record lookup
 		PString number = "h323:" + alias;
 		PString domain = alias.Mid(at+1);
 		if (IsIPAddress(domain)
 			|| (domain.FindRegEx(PRegularExpression(":[0-9]+$", PRegularExpression::Extended)) != P_MAX_INDEX))
 			continue;	// don't use SRV record if domain part is IP or has port (Annex O, O.9), let dns policy handle them
-
+ 
 		// LS Record lookup
 		PStringList ls;
 		if (PDNS::LookupSRV(number, "_h323ls._udp.", ls)) {
@@ -1592,7 +1794,7 @@ Route * SRVPolicy::LSLocalLookup(H225_ArrayOf_AliasAddress & aliases)
 				}
 				PTRACE(4, "ROUTING\tSRV LS located domain " << domain << " at " << ipaddr);
 				H323TransportAddress addr = H323TransportAddress(ipaddr);
-
+ 
 				PIPSocket::Address socketip;
 				WORD port;
 				if (!GetTransportAddress(ipaddr, GK_DEF_UNICAST_RAS_PORT, socketip, port) && socketip.IsValid()) {
@@ -1612,6 +1814,9 @@ Route * SRVPolicy::LSLocalLookup(H225_ArrayOf_AliasAddress & aliases)
 						Route * route = new Route("srv", ep);
 						return route;
 					}
+				} else if (m_resolveNonLocalLRQs) {
+					// just rewrite the destination based on the SRV record and let the following policies handle it
+					H323SetAliasAddress(number, aliases[0]);
 				} else {
 					PTRACE(3, "ROUTING\tSkipped, using only local LS for LRQs");
 				}
@@ -1620,7 +1825,7 @@ Route * SRVPolicy::LSLocalLookup(H225_ArrayOf_AliasAddress & aliases)
 	}
 	return NULL;
 }
-
+ 
 Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
 {
 	for (PINDEX i = 0; i < aliases.GetSize(); ++i) {
@@ -1633,7 +1838,7 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
 		// skip empty aliases or those without at-sign
 	    if ((alias.GetLength() == 0) || (at == P_MAX_INDEX))
 			continue;
-
+ 
 		// DNS SRV Record lookup
 		PString number = "h323:" + alias;
 		PString domain = alias.Mid(at+1);
@@ -1652,8 +1857,11 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
 					PTRACE(1, "ROUTING\tERROR in CS SRV lookup (" << cs[j] << ")");
 					continue;
 				}
+				PStringArray parts = SplitIPAndPort(dom, GK_DEF_ENDPOINT_SIGNAL_PORT);
+				dom = parts[0];
+				WORD port = (WORD)parts[1].AsUnsigned();
 				PTRACE(4, "ROUTING\tSRV CS converted remote party " << alias << " to " << cs[j]);
-				if (GetTransportAddress(dom, GK_DEF_ENDPOINT_SIGNAL_PORT, dest)) {
+				if (GetTransportAddress(dom, port, dest)) {
 					PIPSocket::Address addr;
 					if (!(GetIPFromTransportAddr(dest, addr) && addr.IsValid()))
 						continue;
@@ -1683,7 +1891,7 @@ Route * SRVPolicy::CSLookup(H225_ArrayOf_AliasAddress & aliases, bool localonly)
 	}
 	return NULL;
 }
-
+ 
 // used for ARQs and Setups
 bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
@@ -1710,10 +1918,10 @@ bool SRVPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddres
 	return false;
 }
 
-// usd for LRQs
+// used for LRQs
 bool SRVPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
 { 
-	Route * route = CSLookup(aliases, true);
+	Route * route = CSLookup(aliases, !m_resolveNonLocalLRQs);
 	if (route) {
 		if (route->m_flags & Route::e_Reject) {
 			request.SetFlag(RoutingRequest::e_Reject);
@@ -1737,26 +1945,29 @@ bool SRVPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddre
 }
 #endif
 
+
 ///////////////////////////////////////////////////////////////////////////////////////////
 // RDS policy
 #ifdef hasRDS
 
 class RDSPolicy : public AliasesPolicy {
 public:
-	RDSPolicy() { m_name = "RDS"; }
+	RDSPolicy();
 
 protected:
-    virtual bool OnRequest(SetupRequest &) { return false; }
-    virtual bool OnRequest(FacilityRequest &) { return false; }
-
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
+
+	bool m_resolveLRQs;
 };
 
-bool RDSPolicy::FindByAliases(
-	RoutingRequest &request,
-	H225_ArrayOf_AliasAddress &aliases
-	)
+RDSPolicy::RDSPolicy()
+{
+	m_name = "RDS";
+	m_resolveLRQs = Toolkit::AsBool(GkConfig()->GetString("Routing::RDS", "ResolveLRQ", "0"));
+}
+
+bool RDSPolicy::FindByAliases(RoutingRequest & request, H225_ArrayOf_AliasAddress & aliases)
 {
 	for (PINDEX a = 0; a < aliases.GetSize(); ++a) {
 		PString alias(AsString(aliases[a], FALSE));
@@ -1820,11 +2031,14 @@ bool RDSPolicy::FindByAliases(
 	return false;
 }
 
-
-bool RDSPolicy::FindByAliases(LocationRequest & /* request */, H225_ArrayOf_AliasAddress & /* aliases */)
-{ 
-    PTRACE(4, "ROUTING\tPolicy RDS not supported for LRQ");
-	return false;
+bool RDSPolicy::FindByAliases(LocationRequest & request, H225_ArrayOf_AliasAddress & aliases)
+{
+	if (m_resolveLRQs) {
+		return RDSPolicy::FindByAliases((RoutingRequest&)request, aliases);
+	} else {
+		PTRACE(4, "ROUTING\tPolicy RDS configured not to resolve LRQs");
+		return false;
+	}
 }
 #endif
 
