@@ -5,7 +5,7 @@
 // Routing Mechanism for GNU Gatekeeper
 //
 // Copyright (c) Citron Network Inc. 2003
-// Copyright (c) 2004-2010, Jan Willamowius
+// Copyright (c) 2004-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -15,7 +15,7 @@
 //////////////////////////////////////////////////////////////////
 
 #ifndef ROUTING_H
-#define ROUTING_H "@(#) $Id: Routing.h,v 1.54 2011/07/25 09:10:21 willamowius Exp $"
+#define ROUTING_H "@(#) $Id: Routing.h,v 1.60 2011/12/05 21:33:51 willamowius Exp $"
 
 #include <map>
 #include <list>
@@ -142,11 +142,11 @@ public:
 	typedef R ReqObj;
 	typedef W Wrapper;
 
-	Request(ReqObj & r, Wrapper *w) : m_request(r), m_wrapper(w) {}
+	Request(ReqObj & r, Wrapper *w) : m_request(r), m_wrapper(w), m_clientAuthId(0) {}
 	Request(ReqObj & r, Wrapper *w, const PString & id, PUInt64 authid)
 		: m_request(r), m_wrapper(w), m_callingStationId(id), m_clientAuthId(authid) {}
 	Request(ReqObj & r, Wrapper *w, const std::list<Route> &failedRoutes)
-		: RoutingRequest(failedRoutes), m_request(r), m_wrapper(w) {}
+		: RoutingRequest(failedRoutes), m_request(r), m_wrapper(w), m_clientAuthId(0) {}
 
 	bool Process();
 
@@ -183,20 +183,14 @@ public:
 	template <class R> bool HandleRas(Request<R,RasMsg> & request)
 	{
 		if( IsActive() ) {
-#if PTRACING
 			const char* tagname = request.GetWrapper()
 				? request.GetWrapper()->GetTagName() : "unknown";
 			const unsigned seqnum = request.GetRequest().m_requestSeqNum.GetValue();
 			PTRACE(5,"ROUTING\tChecking policy "<<m_name
-				<<" for the request "<<tagname<<' '<<seqnum
-				);
-#endif
+				<<" for the request "<<tagname<<' '<<seqnum);
 			if( OnRequest(request) ) {
-#if PTRACING
 				PTRACE(5,"ROUTING\tPolicy "<<m_name
-					<<" applied to the request "<<tagname<<' '<<seqnum
-					);
-#endif
+					<<" applied to the request "<<tagname<<' '<<seqnum);
 				return true;
 			}
 		}
@@ -231,7 +225,9 @@ class ExplicitPolicy : public Policy {
 public:
 	ExplicitPolicy();
 
-	static void MapDestination(H225_TransportAddress & addr);
+	static void MapDestination(H225_AdmissionRequest & arq);
+	static void MapDestination(H225_Setup_UUIE & setupBody);
+	static void MapDestination(H225_Facility_UUIE & facility);
 	static void OnReload();
 
 protected:
@@ -240,7 +236,7 @@ protected:
 	virtual bool OnRequest(SetupRequest &);
 	virtual bool OnRequest(FacilityRequest &);
 
-	static std::map<PString, H225_TransportAddress> m_destMap;
+	static std::map<PString, PString> m_destMap;
 };
 
 
@@ -329,6 +325,8 @@ protected:
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
 
+	virtual bool DNSLookup(const PString & hostname, PIPSocket::Address & addr) const;
+
 	static bool m_resolveNonLocalLRQs;
 };
 
@@ -378,10 +376,13 @@ private:
 // a policy to look up the destination from ENUM Name Server
 class ENUMPolicy : public AliasesPolicy {
 public:
-	ENUMPolicy() { m_name = "ENUM"; }
+	ENUMPolicy();
+
 protected:
 	virtual bool FindByAliases(RoutingRequest &, H225_ArrayOf_AliasAddress &);
 	virtual bool FindByAliases(LocationRequest &, H225_ArrayOf_AliasAddress &);
+
+	bool m_resolveLRQs;
 };
 
 class DestinationRoutes {
