@@ -13,7 +13,7 @@
 
 
 #ifndef GK_H
-#define GK_H "@(#) $Id: gk.h,v 1.36 2010/10/25 15:01:51 willamowius Exp $"
+#define GK_H "@(#) $Id: gk.h,v 1.39 2011/11/28 12:33:41 willamowius Exp $"
 
 #include "config.h"
 #include "version.h"
@@ -59,10 +59,9 @@ class Gatekeeper : public PProcess
 	virtual void Terminate();
 	virtual PBoolean OnPause();
 	virtual void OnContinue();
-    virtual void OnControl();
+	virtual void OnControl();
 #endif
 
-#if PTRACING
 	enum RotationIntervals {
 		Hourly,
 		Daily,
@@ -79,18 +78,15 @@ class Gatekeeper : public PProcess
 	static bool ReopenLogFile();	
 	static void CloseLogFile();
 
-	static void EnableLogFileRotation(
-		bool enable = true
-		);
+	static void EnableLogFileRotation(bool enable = true);
 
 	/** Rotate the log file, saving old file contents to a different
 	    file and starting with a new one. This is a callback function
 	    called when the rotation timer expires.
 	*/
 	static void RotateOnTimer(
-		GkTimer* timer /// timer object that triggered rotation
+		GkTimer * timer /// timer object that triggered rotation
 		);
-#endif // PTRACING
 
  protected:
 	/** returns the template string for which the cmommand line is parsed */
@@ -104,34 +100,31 @@ class Gatekeeper : public PProcess
 	//@{
 
 	/** installs the signal handlers; First called init method. */
-	virtual bool InitHandlers(const PArgList &args);
+	virtual bool InitHandlers(const PArgList & args);
 
-	virtual bool InitConfig(const PArgList &args);
+	virtual bool InitConfig(const PArgList & args);
 
 	/** initiates logging and tracing; Called after #InitConfig# */
-	virtual bool InitLogging(const PArgList &args);
+	virtual bool InitLogging(const PArgList & args);
 
 	/** print the available command-line-options **/
-	void PrintOpts(void);
+	void PrintOpts();
 
 	/** Set a new user and group (ownership) for the GK process.
 		The group that will be set is the user's default group.
 	*/
-	virtual bool SetUserAndGroup(const PString &username);
+	virtual bool SetUserAndGroup(const PString & username);
 
 	//@}
 
 private:
-#if PTRACING
 	/// parse rotation interval from the config
 	static void GetRotateInterval(
-		PConfig& cfg, /// the config
-		const PString& section /// name of the config section to check
+		PConfig & cfg, /// the config
+		const PString & section /// name of the config section to check
 		);
-#endif
 
 private:
-#if PTRACING
 	/// rotate file after the specified period of time (if >= 0)
 	static int m_rotateInterval;
 	/// a minute when the interval based rotation should occur
@@ -150,7 +143,6 @@ private:
 	static PMutex m_logFileMutex;
 	/// human readable names for rotation intervals
 	static const char* const m_intervalNames[];
-#endif // PTRACING
 
 #ifdef COMPILE_AS_SERVICE
 	PString savedArguments;
