@@ -3,7 +3,7 @@
  *
  * IP based authentication modules
  *
- * @(#) $Id: ipauth.cxx,v 1.12 2011/07/20 14:06:46 willamowius Exp $
+ * @(#) $Id: ipauth.cxx,v 1.14 2011/11/28 12:33:41 willamowius Exp $
  *
  * Copyright (c) 2005, Michal Zygmuntowicz
  * Copyright (c) 2006-2010, Jan Willamowius
@@ -35,8 +35,8 @@ public:
 	void AddPrefix(const PString &);
 	void SortPrefix(bool greater = true);
 	int PrefixMatch(const PString &) const;
-	std::string PrintOn(void) const;
-	std::string PrintPrefix(void) const;
+	std::string PrintOn() const;
+	std::string PrintPrefix() const;
 
 	IPAuthPrefix& operator=(const IPAuthPrefix&);
 	IPAuthPrefix& operator=(bool);
@@ -226,7 +226,6 @@ FileIPAuth::FileIPAuth(
 	
 	PTRACE(5, GetName() << "\t" << m_authList.size() << " entries loaded");
 
-#if PTRACING
 	if (PTrace::CanTrace(6)) {
 		ostream &strm = PTrace::Begin(6, __FILE__, __LINE__);
 		strm << GetName() << " entries:\n";
@@ -239,7 +238,6 @@ FileIPAuth::FileIPAuth(
 		}
 		PTrace::End(strm);
 	}
-#endif
 
 	if (dynamicCfg) {
 		delete cfg;
@@ -366,7 +364,7 @@ int IPAuthPrefix::PrefixMatch(const PString & number) const
 	return 0;
 }
 
-std::string IPAuthPrefix::PrintOn(void) const
+std::string IPAuthPrefix::PrintOn() const
 {
 	if (!auth)
 		return std::string(" to called any");
@@ -380,7 +378,7 @@ std::string IPAuthPrefix::PrintOn(void) const
 	return ret;
 }
 
-std::string IPAuthPrefix::PrintPrefix(void) const
+std::string IPAuthPrefix::PrintPrefix() const
 {
 	std::string prefix;
 	const_prefix_iterator Iter = Prefixs.begin();
