@@ -2,7 +2,7 @@
 //
 // pwlib_compat.h PWLib compatibility header
 //
-// Copyright (c) 2006-2010, Jan Willamowius
+// Copyright (c) 2006-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -13,7 +13,7 @@
 
 
 #ifndef PWLIB_COMPAT_H
-#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.46 2011/07/20 11:26:53 willamowius Exp $"
+#define PWLIB_COMPAT_H "@(#) $Id: pwlib_compat.h,v 1.51 2011/11/28 12:33:41 willamowius Exp $"
 
 #include "ptbuildopts.h"
 #include "openh323buildopts.h"
@@ -26,6 +26,10 @@
 	#endif
 #endif
 
+#ifndef PTRACING
+	#error "Enable PTRACING in PTLIB to compile GnuGk"
+#endif
+
 // check for PConfig support
 #ifndef P_CONFIG_FILE
 	#error "Make sure PTLib has config file support enabled, avoid --disable-configfile or --enable-openh323 or --enable-minsize etc."
@@ -106,6 +110,15 @@
 	#if PTLIB_MINOR >= 8
 		#define hasDynaLinkGetLastError 1
 	#endif
+	#if PTLIB_MINOR == 9
+		#define hasPTLibTraceOnShutdownBug	1
+	#endif
+	#if PTLIB_MINOR >= 10
+		#ifdef P_HAS_IPV6
+			// IPv6 support before 2.10.0 is too buggy
+			#define hasIPV6     1
+		#endif
+	#endif
 	#if PTLIB_MINOR >= 11
 		#define hasLDAPStartTLS 1
         #define hasNewSTUN      1
@@ -141,6 +154,10 @@
 ///////////////////////////////////////////////
 
 // OpenH323/H323Plus version matching
+#ifdef H323_H235
+   #define HAS_H235_MEDIA 1      // H235 Media Encryption Support
+#endif
+
 #ifdef H323_H460
 	#define HAS_H460 1    // H460 support
 #endif
