@@ -6,7 +6,7 @@
  * Please see docs/radauth.txt for more details.
  *
  * Copyright (c) 2003, Quarcom FHU, Michal Zygmuntowicz
- * Copyright (c) 2005-2010, Jan Willamowius
+ * Copyright (c) 2005-2011, Jan Willamowius
  *
  * This work is published under the GNU Public License version 2 (GPLv2)
  * see file COPYING for details.
@@ -18,7 +18,7 @@
 #if HAS_RADIUS
 
 #ifndef __RADAUTH_H
-#define __RADAUTH_H "@(#) $Id: radauth.h,v 1.19 2010/10/25 15:01:51 willamowius Exp $"
+#define __RADAUTH_H "@(#) $Id: radauth.h,v 1.21 2011/12/07 22:42:32 willamowius Exp $"
 
 #include "gkauth.h"
 
@@ -93,85 +93,7 @@ public:
 		SetupAuthData& authData
 		);
 	
-protected:		
-	/** Hook for adding/modifying pdu before it is sent.
-		It can be used to add custom attributes, for example.
-		
-		@return
-		true if PDU should be sent, false to reject RRQ
-		(rejectReason can be set to indicate a particular reason).
-	*/
-	virtual bool OnSendPDU(
-		RadiusPDU& pdu, /// PDU to be sent
-		RasPDU<H225_RegistrationRequest>& rrqPdu, /// RRQ being processed
-		RRQAuthData& authData /// authorization data
-		);
-
-	/** Hook for adding/modifying pdu before it is sent.
-		It can be used to add custom attributes, for example.
-		
-		@return
-		TRUE if PDU should be sent, FALSE to reject ARQ
-		(rejectReason can be set to indicate a particular reason).
-	*/
-	virtual bool OnSendPDU(
-		RadiusPDU& pdu, /// PDU to be sent
-		RasPDU<H225_AdmissionRequest>& arqPdu, /// ARQ being processed
-		ARQAuthData& authData /// authorization data 
-		);
-
-	/** Hook for adding/modifying pdu before it is sent.
-		It can be used to add custom attributes, for example.
-		
-		@return
-		TRUE if PDU should be sent, FALSE to reject the call
-		(rejectReason can be set to indicate a particular reason).
-	*/
-	virtual bool OnSendPDU(
-		RadiusPDU &pdu, /// PDU to be sent
-		SetupMsg &setup, /// Q.931/H.225 Setup being processed
-		SetupAuthData &authData /// authorization data 
-		);
-		
-	/** Hook for processing pdu after it is received.
-		It can be used to process custom attributes, for example.
-		
-		@return
-		TRUE if PDU should be accepted, FALSE to reject RRQ
-		(rejectReason can be set to indicate a particular reason).
-	*/
-	virtual bool OnReceivedPDU(
-		RadiusPDU& pdu, /// received PDU 
-		RasPDU<H225_RegistrationRequest>& rrqPdu, /// RRQ being processed
-		RRQAuthData& authData /// authorization data
-		);
-
-	/** Hook for processing pdu after it is received.
-		It can be used to process custom attributes, for example.
-		
-		@return
-		TRUE if PDU should be accepted, FALSE to reject ARQ
-		(rejectReason can be set to indicate a particular reason).
-	*/
-	virtual bool OnReceivedPDU(
-		RadiusPDU& pdu, /// received PDU
-		RasPDU<H225_AdmissionRequest>& arqPdu, /// ARQ being processed
-		ARQAuthData& authData /// authorization data 
-		);
-	
-	/** Hook for processing pdu after it is received.
-		It can be used to process custom attributes, for example.
-		
-		@return
-		TRUE if PDU should be accepted, FALSE to reject the call
-		(rejectReason can be set to indicate a particular reason).
-	*/
-	virtual bool OnReceivedPDU(
-		RadiusPDU &pdu, /// received PDU
-		SetupMsg &setup, /// Q.931/H.225 Setup being processed
-		SetupAuthData &authData /// authorization data 
-		);
-	
+protected:
 	/** Hook for appending username/password attributes 
 		proper for derived authenticators.
 		
