@@ -3,7 +3,7 @@
 // yasocket.cxx
 //
 // Copyright (c) Citron Network Inc. 2002-2003
-// Copyright (c) 2004-2010, Jan Willamowius
+// Copyright (c) 2004-2011, Jan Willamowius
 //
 // This work is published under the GNU Public License version 2 (GPLv2)
 // see file COPYING for details.
@@ -54,7 +54,7 @@ bool YaSelectList::Select(SelectType t, const PTimeInterval & timeout)
 
 	const unsigned long msec = timeout.GetInterval();
 	struct timeval tval;
-	tval.tv_sec  = msec / 1000;
+	tval.tv_sec = msec / 1000;
 	tval.tv_usec = (msec - tval.tv_sec * 1000) * 1000;
 	int r = ::select(maxfd + 1, readfds, writefds, 0, &tval);
 	if (r > 0) {
@@ -148,22 +148,20 @@ bool YaSocket::Close()
 	return true;
 }
 
-bool YaSocket::Read(void *buf, int sz)
+bool YaSocket::Read(void * buf, int sz)
 {
 	int r = os_recv(buf, sz);
 	lastReadCount = ConvertOSError(r, PSocket::LastReadError) ? r : 0;
 	return lastReadCount > 0;
 }
 
-bool YaSocket::ReadBlock(void *buf, int len)
+bool YaSocket::ReadBlock(void * buf, int len)
 {
 	// lazy implementation, but it is enough for us...
 	return Read(buf, len) && lastReadCount == len;
 }
 
-bool YaSocket::CanRead(
-	long timeout
-	) const
+bool YaSocket::CanRead(long timeout) const
 {
 	const int h = os_handle;
 	if (h < 0)
@@ -173,14 +171,12 @@ bool YaSocket::CanRead(
 	fdset.add(h);
 
 	struct timeval tval;
-	tval.tv_sec  = timeout / 1000;
+	tval.tv_sec = timeout / 1000;
 	tval.tv_usec = (timeout - tval.tv_sec * 1000) * 1000;
 	return ::select(h + 1, (fd_set*)fdset, NULL, NULL, &tval) > 0;
 }
 
-bool YaSocket::CanWrite(
-	long timeout
-	) const
+bool YaSocket::CanWrite(long timeout) const
 {
 	const int h = os_handle;
 	if (h < 0)
@@ -190,12 +186,12 @@ bool YaSocket::CanWrite(
 	fdset.add(h);
 
 	struct timeval tval;
-	tval.tv_sec  = timeout / 1000;
+	tval.tv_sec = timeout / 1000;
 	tval.tv_usec = (timeout - tval.tv_sec * 1000) * 1000;
 	return ::select(h + 1, NULL, (fd_set*)fdset, NULL, &tval) > 0;
 }
 
-bool YaSocket::Write(const void *buf, int sz)
+bool YaSocket::Write(const void * buf, int sz)
 {
 	lastWriteCount = 0;
 	if (!CanWrite(writeTimeout.GetInterval())) {
@@ -216,11 +212,23 @@ PBoolean YaSocket::GetLocalAddress(Address & addr) const
 
 PBoolean YaSocket::GetLocalAddress(Address & addr, WORD & pt) const
 {
+#ifdef hasIPV6
+	sockaddr_in6 inaddr;
+#else
 	sockaddr_in inaddr;
+#endif
 	socklen_t insize = sizeof(inaddr);
 	if (::getsockname(os_handle, (struct sockaddr*)&inaddr, &insize) == 0) {
-		addr = inaddr.sin_addr;
-		pt = ntohs(inaddr.sin_port);
+#ifdef hasIPV6
+		if (((struct sockaddr*)&inaddr)->sa_family == AF_INET6) {
+			addr = ((struct sockaddr_in6*)&inaddr)->sin6_addr;
+			pt = ntohs(((struct sockaddr_in6*)&inaddr)->sin6_port);
+		} else
+#endif
+		{
+			addr = ((struct sockaddr_in*)&inaddr)->sin_addr;
+			pt = ntohs(((struct sockaddr_in*)&inaddr)->sin_port);
+		}
 	}
 	return true;
 }
@@ -280,16 +288,28 @@ bool YaSocket::SetNonBlockingMode()
 bool YaSocket::Bind(const Address & addr, WORD pt)
 {
 	if (IsOpen()) {
-		sockaddr_in inaddr;
-		memset(&inaddr, 0, sizeof(inaddr));
-		inaddr.sin_family = AF_INET;
-		inaddr.sin_addr.s_addr = addr;
-		inaddr.sin_port = htons(pt);
-		if (ConvertOSError(::bind(os_handle, (struct sockaddr *)&inaddr, sizeof(inaddr)))) {
-			socklen_t insize = sizeof(inaddr);
-			if (::getsockname(os_handle, (struct sockaddr *)&inaddr, &insize) == 0) {
-				port = ntohs(inaddr.sin_port);
-				return true;
+#ifdef hasIPV6
+		if (addr.GetVersion() == 6) {
+			struct sockaddr_in6 inaddr;
+			SetSockaddr(inaddr, addr, pt);
+			if (ConvertOSError(::bind(os_handle, (struct sockaddr*)&inaddr, sizeof(inaddr)))) {
+				socklen_t insize = sizeof(inaddr);
+				if (::getsockname(os_handle, (struct sockaddr*)&inaddr, &insize) == 0) {
+					port = ntohs(inaddr.sin6_port);
+					return true;
+				}
+			}
+		} else
+#endif
+		{
+			struct sockaddr_in inaddr;
+			SetSockaddr(inaddr, addr, pt);
+			if (ConvertOSError(::bind(os_handle, (struct sockaddr*)&inaddr, sizeof(inaddr)))) {
+				socklen_t insize = sizeof(inaddr);
+				if (::getsockname(os_handle, (struct sockaddr*)&inaddr, &insize) == 0) {
+					port = ntohs(inaddr.sin_port);
+					return true;
+				}
 			}
 		}
 	}
@@ -300,19 +320,32 @@ bool YaSocket::Bind(const Address & addr, WORD pt)
 // class YaTCPSocket
 YaTCPSocket::YaTCPSocket(WORD pt)
 {
-	peeraddr.sin_family = AF_INET;
+	((struct sockaddr*)&peeraddr)->sa_family = AF_INET;		// overwritten in Connect()
 	SetPort(pt);
 }
 
 void YaTCPSocket::GetPeerAddress(Address & addr) const
 {
-	addr = peeraddr.sin_addr;
+#ifdef hasIPV6
+	if (((struct sockaddr*)&peeraddr)->sa_family == AF_INET6)
+		addr = ((struct sockaddr_in6*)&peeraddr)->sin6_addr;
+	else
+#endif
+		addr = ((struct sockaddr_in*)&peeraddr)->sin_addr;
 }
 
 void YaTCPSocket::GetPeerAddress(Address & addr, WORD & pt) const
 {
-	addr = peeraddr.sin_addr;
-	pt = ntohs(peeraddr.sin_port);
+#ifdef hasIPV6
+	if (((struct sockaddr*)&peeraddr)->sa_family == AF_INET6) {
+		addr = ((struct sockaddr_in6*)&peeraddr)->sin6_addr;
+		pt = ntohs(((struct sockaddr_in6*)&peeraddr)->sin6_port);
+	} else
+#endif
+	{
+		addr = ((struct sockaddr_in*)&peeraddr)->sin_addr;
+		pt = ntohs(((struct sockaddr_in*)&peeraddr)->sin_port);
+	}
 }
 
 bool YaTCPSocket::SetLinger()
@@ -324,12 +357,23 @@ bool YaTCPSocket::SetLinger()
 
 bool YaTCPSocket::Listen(unsigned qs, WORD pt, PSocket::Reusability reuse)
 {
-	return Listen(INADDR_ANY, qs, pt, reuse);
+	return Listen(GNUGK_INADDR_ANY, qs, pt, reuse);
 }
 
 bool YaTCPSocket::Listen(const Address & addr, unsigned qs, WORD pt, PSocket::Reusability reuse)
 {
-	os_handle = ::socket(PF_INET, SOCK_STREAM, 0);
+#ifdef hasIPV6
+	if (addr.GetVersion() == 6) {
+		os_handle = ::socket(PF_INET6, SOCK_STREAM, 0);
+		if (addr.IsAny() && !SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
+			PTRACE(1, "Removing of IPV6_V6ONLY failed");
+		}
+	}
+	else
+#endif
+	{
+		os_handle = ::socket(PF_INET, SOCK_STREAM, 0);
+	}
 	if (!ConvertOSError(os_handle))
 		return false;
 
@@ -343,6 +387,14 @@ bool YaTCPSocket::Listen(const Address & addr, unsigned qs, WORD pt, PSocket::Re
 	return false;
 }
 
+#ifdef hasIPV6
+bool YaTCPSocket::DualStackListen(WORD port)
+{
+	return Listen(GNUGK_INADDR_ANY, 1, port, PSocket::CanReuseAddress);
+}
+#endif
+
+
 bool YaTCPSocket::Accept(YaTCPSocket & socket)
 {
 	while (true) {
@@ -369,7 +421,12 @@ bool YaTCPSocket::Accept(YaTCPSocket & socket)
 		SetLinger();
 		SetNonBlockingMode();
 		SetWriteTimeout(PTimeInterval(10));
-		SetName(AsString(peeraddr.sin_addr, ntohs(peeraddr.sin_port)));
+#ifdef hasIPV6
+		if (((struct sockaddr *)&peeraddr)->sa_family == AF_INET6)
+			SetName(AsString(((struct sockaddr_in6*)&peeraddr)->sin6_addr, ntohs(((struct sockaddr_in6*)&peeraddr)->sin6_port)));
+		else
+#endif
+			SetName(AsString(((struct sockaddr_in*)&peeraddr)->sin_addr, ntohs(((struct sockaddr_in*)&peeraddr)->sin_port)));
 		port = socket.GetPort();
 		return true;
 	}
@@ -379,30 +436,41 @@ bool YaTCPSocket::Accept(YaTCPSocket & socket)
 bool YaTCPSocket::Connect(const Address & iface, WORD localPort, const Address & addr)
 {
 	if (os_handle < 0) {
-		os_handle = ::socket(PF_INET, SOCK_STREAM, 0);
+#ifdef hasIPV6
+		if (iface.GetVersion() == 6)
+			os_handle = ::socket(PF_INET6, SOCK_STREAM, 0);
+		else
+#endif
+			os_handle = ::socket(PF_INET, SOCK_STREAM, 0);
 		if (!ConvertOSError(os_handle))
 			return false;
 	}
 
 	SetOption(SO_REUSEADDR, 0);
-	
+
 	int optval;
 	socklen_t optlen = sizeof(optval);
 
 	WORD peerPort = port;
 	// bind local interface and port
-	if (iface != INADDR_ANY || localPort != 0)
+	if (!iface.IsAny() || localPort != 0)
 		if (!Bind(iface, localPort))
 			return false;
 
 	// connect in non-blocking mode
 	SetNonBlockingMode();
 	SetWriteTimeout(PTimeInterval(10));
-	peeraddr.sin_addr = addr;
-	peeraddr.sin_port = htons(port = peerPort);
+	port = peerPort;
+	SetSockaddr(peeraddr, addr, peerPort);
 	SetName(AsString(addr, port));
 
-	int r = ::connect(os_handle, (struct sockaddr *)&peeraddr, sizeof(peeraddr));
+#ifdef P_FREEBSD
+	// FreeBSD is extra picky about the size argument, not sure if thats the correct size for Linux
+	int r = ::connect(os_handle, (struct sockaddr*)&peeraddr, sizeof(struct sockaddr));
+#else
+	int r = ::connect(os_handle, (struct sockaddr*)&peeraddr, sizeof(peeraddr));
+#endif
+
 #ifdef _WIN32
 	if ((r != 0) && (WSAGetLastError() != WSAEWOULDBLOCK))
 #else
@@ -434,10 +502,10 @@ bool YaTCPSocket::Connect(const Address & iface, WORD localPort, const Address &
 
 bool YaTCPSocket::Connect(const Address & addr)
 {
-	return YaTCPSocket::Connect(INADDR_ANY, 0, addr);
+	return YaTCPSocket::Connect(GNUGK_INADDR_ANY, 0, addr);
 }
 
-int YaTCPSocket::os_recv(void *buf, int sz)
+int YaTCPSocket::os_recv(void * buf, int sz)
 {
 #if HAS_MSG_NOSIGNAL
 	return ::recv(os_handle, buf, sz, MSG_NOSIGNAL);
@@ -446,7 +514,7 @@ int YaTCPSocket::os_recv(void *buf, int sz)
 #endif
 }
 
-int YaTCPSocket::os_send(const void *buf, int sz)
+int YaTCPSocket::os_send(const void * buf, int sz)
 {
 #if HAS_MSG_NOSIGNAL
 	return ::send(os_handle, buf, sz, MSG_NOSIGNAL);
@@ -457,20 +525,28 @@ int YaTCPSocket::os_send(const void *buf, int sz)
 
 
 // class YaUDPSocket
-YaUDPSocket::YaUDPSocket()
+YaUDPSocket::YaUDPSocket(WORD port, int iAddressFamily)
 {
-	sendaddr.sin_family = AF_INET;
-	sendaddr.sin_port = 0;
+	((struct sockaddr*)&sendaddr)->sa_family = iAddressFamily;
+	((struct sockaddr_in*)&sendaddr)->sin_port = htons(port);
 }
 
 bool YaUDPSocket::Listen(unsigned, WORD pt, PSocket::Reusability reuse)
 {
-	return Listen(INADDR_ANY, 0, pt, reuse);
+	return Listen(GNUGK_INADDR_ANY, 0, pt, reuse);
 }
 
 bool YaUDPSocket::Listen(const Address & addr, unsigned, WORD pt, PSocket::Reusability reuse)
 {
-	os_handle = ::socket(PF_INET, SOCK_DGRAM, 0);
+#ifdef hasIPV6
+	if (addr.GetVersion() == 6) {
+		os_handle = ::socket(PF_INET6, SOCK_DGRAM, 0);
+		if (addr.IsAny() && !SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
+			PTRACE(1, "Removing of IPV6_V6ONLY failed");
+		}
+	} else
+#endif
+		os_handle = ::socket(PF_INET, SOCK_DGRAM, 0);
 	if (!ConvertOSError(os_handle))
 		return false;
 
@@ -481,28 +557,50 @@ bool YaUDPSocket::Listen(const Address & addr, unsigned, WORD pt, PSocket::Reusa
 	return Bind(addr, pt);
 }
 
+#ifdef hasIPV6
+bool YaUDPSocket::DualStackListen(const Address & addr, WORD pt)
+{
+	return Listen(addr, 0, pt, PSocket::CanReuseAddress);
+}
+#endif
+
 void YaUDPSocket::GetLastReceiveAddress(Address & addr, WORD & pt) const
 {
-	addr = recvaddr.sin_addr;
-	pt = ntohs(recvaddr.sin_port);
+#ifdef hasIPV6
+	if (((struct sockaddr*)&recvaddr)->sa_family == AF_INET6) {
+		addr = ((struct sockaddr_in6*)&recvaddr)->sin6_addr;
+		pt = ntohs(((struct sockaddr_in6*)&recvaddr)->sin6_port);
+	} else
+#endif
+	{
+		addr = ((struct sockaddr_in*)&recvaddr)->sin_addr;
+		pt = ntohs(((struct sockaddr_in*)&recvaddr)->sin_port);
+	}
 }
 
 void YaUDPSocket::SetSendAddress(const Address & addr, WORD pt)
 {
-	sendaddr.sin_addr = addr;
-	sendaddr.sin_port = htons(pt);
+	SetSockaddr(sendaddr, addr, pt);
 }
 
 void YaUDPSocket::GetSendAddress(
-	Address& address, /// IP address to send packets.
-	WORD& port /// Port to send packets.
+	Address & address, /// IP address to send packets.
+	WORD & port /// Port to send packets.
 	)
 {
-	address = sendaddr.sin_addr;
-	port = ntohs(sendaddr.sin_port);
+#ifdef hasIPV6
+	if (((struct sockaddr*)&sendaddr)->sa_family == AF_INET6) {
+		address = ((struct sockaddr_in6*)&sendaddr)->sin6_addr;
+		port = ntohs(((struct sockaddr_in6*)&sendaddr)->sin6_port);
+	} else
+#endif
+	{
+		address = ((struct sockaddr_in*)&sendaddr)->sin_addr;
+		port = ntohs(((struct sockaddr_in*)&sendaddr)->sin_port);
+	}
 }
 
-bool YaUDPSocket::ReadFrom(void *buf, PINDEX len, Address & addr, WORD pt)
+bool YaUDPSocket::ReadFrom(void * buf, PINDEX len, Address & addr, WORD pt)
 {
 	bool result = Read(buf, len);
 	if (result)
@@ -510,25 +608,145 @@ bool YaUDPSocket::ReadFrom(void *buf, PINDEX len, Address & addr, WORD pt)
 	return result;
 }
 
-bool YaUDPSocket::WriteTo(const void *buf, PINDEX len, const Address & addr, WORD pt)
+bool YaUDPSocket::WriteTo(const void * buf, PINDEX len, const Address & addr, WORD pt)
 {
 	SetSendAddress(addr, pt);
 	return Write(buf, len);
 }
 
-int YaUDPSocket::os_recv(void *buf, int sz)
+int YaUDPSocket::os_recv(void * buf, int sz)
 {
 	socklen_t addrlen = sizeof(recvaddr);
 	return ::recvfrom(os_handle, buf, sz, 0, (struct sockaddr *)&recvaddr, &addrlen);
 }
 
-int YaUDPSocket::os_send(const void *buf, int sz)
+int YaUDPSocket::os_send(const void * buf, int sz)
 {
 	return ::sendto(os_handle, buf, sz, 0, (struct sockaddr *)&sendaddr, sizeof(sendaddr));
 }
 
 #else // LARGE_FDSET
 
+#ifdef hasIPV6
+bool TCPSocket::DualStackListen(WORD newPort)
+{
+	if (!Toolkit::Instance()->IsIPv6Enabled())
+		return Listen(GNUGK_INADDR_ANY, 1, newPort, PSocket::CanReuseAddress);
+
+	// make sure we have a port
+	if (newPort != 0)
+		port = newPort;
+
+	// Always close and re-open as the bindAddr address family might change.
+	os_close();
+
+	// attempt to create a socket
+	if (!OpenSocket(PF_INET6)) {
+		PTRACE(4, "Socket\tOpenSocket failed");
+		return false;
+	}
+
+	// allow IPv4 connects
+	if (!SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
+		PTRACE(4, "Socket\tSetOption(IPV6_V6ONLY) failed");
+	}
+
+	// attempt to listen
+	if (!SetOption(SO_REUSEADDR, 1)) {
+		PTRACE(4, "Socket\tSetOption(SO_REUSEADDR) failed");
+		os_close();
+		return false;
+	}
+
+	sockaddr_in6 sa;
+	memset(&sa, 0, sizeof(sa));
+	sa.sin6_family = AF_INET6;
+	sa.sin6_addr = in6addr_any;
+	sa.sin6_port = htons(newPort);
+	if (!ConvertOSError(::bind(os_handle, (sockaddr*)&sa, sizeof(sa)))) {
+		os_close();
+		return false;
+	}
+
+	if (!ConvertOSError(::listen(os_handle, 1))) {
+		PTRACE(4, "Socket\tlisten failed: " << GetErrorText());
+		os_close();
+		return false;
+	}
+
+	if (port != 0)
+		return true;
+
+	socklen_t size = sizeof(sa);
+	if (!ConvertOSError(::getsockname(os_handle, (sockaddr*)&sa, &size))) {
+		PTRACE(4, "Socket\tgetsockname failed: " << GetErrorText());
+		os_close();
+		return false;
+	}
+
+	port = ntohs(sa.sin6_port);
+	return true;
+}
+#endif
+
+UDPSocket::UDPSocket(WORD port, int iAddressFamily)
+#ifdef hasIPV6
+	// can use always, because old PTLib versions (eg. 2.4.5) didn't have 2nd argument
+	: PUDPSocket(port, iAddressFamily)
+#endif
+{
+}
+
+#ifdef hasIPV6
+bool UDPSocket::DualStackListen(const PIPSocket::Address & localAddr, WORD newPort)
+{
+	if (!Toolkit::Instance()->IsIPv6Enabled() || !localAddr.IsAny())
+		return Listen(localAddr, 0, newPort);
+
+	// make sure we have a port
+	if (newPort != 0)
+		port = newPort;
+
+	// Always close and re-open as the bindAddr address family might change.
+	os_close();
+
+	// attempt to create a socket
+	if (!OpenSocket(PF_INET6)) {
+		PTRACE(4, "Socket\tOpenSocket failed");
+		return false;
+	}
+
+	// allow IPv4 connects
+	if (!SetOption(IPV6_V6ONLY, 0, IPPROTO_IPV6)) {
+		PTRACE(4, "Socket\tSetOption(IPV6_V6ONLY) failed");
+	}
+
+	sockaddr_in6 sa;
+	memset(&sa, 0, sizeof(sa));
+	sa.sin6_family = AF_INET6;
+	sa.sin6_addr = in6addr_any;
+	sa.sin6_port = htons(newPort);
+	if (!ConvertOSError(::bind(os_handle, (sockaddr*)&sa, sizeof(sa)))) {
+		os_close();
+		return false;
+	}
+
+	if (port != 0)
+		return true;
+
+	socklen_t size = sizeof(sa);
+	if (!ConvertOSError(::getsockname(os_handle, (sockaddr*)&sa, &size))) {
+		PTRACE(4, "Socket\tgetsockname failed: " << GetErrorText());
+		os_close();
+		return false;
+	}
+
+	port = ntohs(sa.sin6_port);
+	return true;
+}
+
+#endif
+
 bool SocketSelectList::Select(SelectType t, const PTimeInterval & timeout)
 {
 	if (IsEmpty())
@@ -556,7 +774,7 @@ PSocket *SocketSelectList::operator[](int i) const
 
 
 // class USocket
-USocket::USocket(IPSocket *s, const char *t)
+USocket::USocket(IPSocket * s, const char * t)
 	: self(s), qsize(0), blocked(false), type(t)
 {
 }
@@ -586,7 +804,7 @@ bool USocket::TransmitData(const PString & str)
 	return WriteData(str, str.GetLength());
 }
 
-bool USocket::TransmitData(const BYTE *buf, int len)
+bool USocket::TransmitData(const BYTE * buf, int len)
 {
 	return WriteData(buf, len);
 }
@@ -608,7 +826,7 @@ bool USocket::Flush()
 	return result;
 }
 
-bool USocket::WriteData(const BYTE *buf, int len)
+bool USocket::WriteData(const BYTE * buf, int len)
 {
 	if (!IsSocketOpen())
 		return false;
@@ -671,7 +889,7 @@ bool USocket::ErrorHandler(PSocket::ErrorGroup group)
 	return false;
 }
 
-bool USocket::InternalWriteData(const BYTE *buf, int len)
+bool USocket::InternalWriteData(const BYTE * buf, int len)
 {
 	if (self->Write(buf, len)) {
 		PTRACE(6, Name() << ' ' << len << " bytes sent");
@@ -737,7 +955,7 @@ void SocketsReader::Stop()
 	RegularJob::Stop();
 }
 
-void SocketsReader::AddSocket(IPSocket *socket)
+void SocketsReader::AddSocket(IPSocket * socket)
 {
 	if (socket == NULL)
 		return;
@@ -770,21 +988,17 @@ void SocketsReader::CleanUp()
 
 bool SocketsReader::SelectSockets(SocketSelectList & slist)
 {
-#if PTRACING
 	int ss = slist.GetSize();
-#endif
 	ConfigReloadMutex.EndRead();
 	if (!slist.Select(SocketSelectList::Read, m_timeout)) {
 		ConfigReloadMutex.StartRead();
 		return false;
 	}
 	ConfigReloadMutex.StartRead();
-#if PTRACING
 	if (PTrace::CanTrace(6)) {
 		PString msg(PString::Printf, "\t%u sockets selected from %u, total %u/%u", slist.GetSize(), ss, m_socksize, m_rmsize);
 		PTRACE(6, GetName() << msg);
 	}
-#endif
 	return true;
 }
 
@@ -804,25 +1018,7 @@ void SocketsReader::RemoveClosed(bool bDeleteImmediately)
 		m_socksize -= rmsize;
 	}
 }
-/*
-void SocketsReader::RemoveSocket(iterator i)
-{
-	m_sockets.erase(i);
-	--m_socksize;
-	PWaitAndSignal lock(m_rmutex);
-	m_removed.push_back(*i);
-	++m_rmsize;
-}
 
-void SocketsReader::RemoveSocket(IPSocket *s)
-{
-	m_sockets.remove(s);
-	--m_socksize;
-	PWaitAndSignal lock(m_rmutex);
-	m_removed.push_back(s);
-	++m_rmsize;
-}
-*/
 void SocketsReader::Exec()
 {
 	ReadLock cfglock(ConfigReloadMutex);
@@ -861,7 +1057,7 @@ TCPListenSocket::~TCPListenSocket()
 	PTRACE(3, "TCP\tDelete listener " << GetName());
 }
 
-bool TCPListenSocket::IsTimeout(const PTime *now) const
+bool TCPListenSocket::IsTimeout(const PTime * now) const
 {
 	if (readTimeout < PMaxTimeInterval)
 		return IsOpen() ? ((readTimeout > 0) ? ((*now - start) > readTimeout) : false) : true;
@@ -891,7 +1087,7 @@ void TCPServer::LoadConfig()
 	}
 }
 
-bool TCPServer::CloseListener(TCPListenSocket *socket)
+bool TCPServer::CloseListener(TCPListenSocket * socket)
 {
 	WriteLock lock(m_listmutex);
 	iterator iter = find(m_sockets.begin(), m_sockets.end(), socket);
@@ -903,7 +1099,7 @@ bool TCPServer::CloseListener(TCPListenSocket *socket)
 		return false;
 }
 
-void TCPServer::ReadSocket(IPSocket *socket)
+void TCPServer::ReadSocket(IPSocket * socket)
 {
 	if (ShutdownMutex.WillBlock()) {
 		PTRACE(4, GetName() << "\tShutdown: Rejecting call on " << socket->GetName());
@@ -925,7 +1121,7 @@ void TCPServer::ReadSocket(IPSocket *socket)
 		one_sec.remove_if(bind2nd(not_equal_to<time_t>(), now));
 		many_sec.remove_if(bind2nd(less<time_t>(), now - check_interval));
 		PTRACE(4, GetName() << "\tcurrent cps=" << one_sec.size() << " calls in interval=" << many_sec.size());
-		if ((many_sec.size() > (cps_limit *  check_interval)) && (one_sec.size() > cps_limit)) {
+		if ((many_sec.size() > (cps_limit * check_interval)) && (one_sec.size() > cps_limit)) {
 			// reject call
 			PTRACE(1, GetName() << "\tRate limit reached (max " << cps_limit << " cps) - rejecting call on " << socket->GetName());
 			int rej = ::accept(socket->GetHandle(), NULL, NULL);
